@@ -21,6 +21,7 @@ from sphinx.domains import Domain, ObjType
 from sphinx.directives import ObjectDescription
 from sphinx.util.nodes import make_refnode
 from sphinx.util.compat import Directive
+from sphinx.util.docfields import Field, GroupedField
 
 
 _identifier_re = re.compile(r'(~?\b[a-zA-Z_][a-zA-Z0-9_]*)\b')
@@ -107,18 +108,15 @@ class DefinitionError(Exception):
     def __init__(self, description):
         self.description = description
 
+    def __str__(self):
+        return unicode(self).encode('utf-8')
+
     def __unicode__(self):
         return self.description
 
-    def __str__(self):
-        return unicode(self.encode('utf-8'))
-
 
 class DefExpr(object):
 
-    def __unicode__(self):
-        raise NotImplementedError()
-
     def __eq__(self, other):
         if type(self) is not type(other):
             return False
@@ -133,17 +131,21 @@ class DefExpr(object):
     def __ne__(self, other):
         return not self.__eq__(other)
 
+    __hash__ = None
+
     def clone(self):
-        """Close a definition expression node"""
+        """Clone a definition expression node."""
         return deepcopy(self)
 
     def get_id(self):
-        """Returns the id for the node"""
+        """Return the id for the node."""
         return u''
 
     def get_name(self):
-        """Returns the name.  Returns either `None` or a node with
-        a name you might call :meth:`split_owner` on.
+        """Return the name.
+
+        Returns either `None` or a node with a name you might call
+        :meth:`split_owner` on.
         """
         return None
 
@@ -156,12 +158,15 @@ class DefExpr(object):
         return None, self
 
     def prefix(self, prefix):
-        """Prefixes a name node (a node returned by :meth:`get_name`)."""
+        """Prefix a name node (a node returned by :meth:`get_name`)."""
         raise NotImplementedError()
 
     def __str__(self):
         return unicode(self).encode('utf-8')
 
+    def __unicode__(self):
+        raise NotImplementedError()
+
     def __repr__(self):
         return '<%s %s>' % (self.__class__.__name__, self)
 
@@ -377,9 +382,9 @@ class NamedDefExpr(DefExpr):
     def get_name(self):
         return self.name.get_name()
 
-    def get_modifiers(self):
+    def get_modifiers(self, visibility='public'):
         rv = []
-        if self.visibility != 'public':
+        if self.visibility != visibility:
             rv.append(self.visibility)
         if self.static:
             rv.append(u'static')
@@ -483,17 +488,24 @@ class FuncDefExpr(NamedDefExpr):
 
 class ClassDefExpr(NamedDefExpr):
 
-    def __init__(self, name, visibility, static):
+    def __init__(self, name, visibility, static, bases):
         NamedDefExpr.__init__(self, name, visibility, static)
+        self.bases = bases
 
     def get_id(self):
         return self.name.get_id()
 
-    def __unicode__(self):
-        buf = self.get_modifiers()
+    def _tostring(self, visibility='public'):
+        buf = self.get_modifiers(visibility)
         buf.append(unicode(self.name))
+        if self.bases:
+            buf.append(u':')
+            buf.append(u', '.join(base._tostring('private')
+                                  for base in self.bases))
         return u' '.join(buf)
 
+    def __unicode__(self):
+        return self._tostring('public')
 
 class DefinitionParser(object):
 
@@ -874,7 +886,21 @@ class DefinitionParser(object):
 
     def parse_class(self):
         visibility, static = self._parse_visibility_static()
-        return ClassDefExpr(self._parse_type(), visibility, static)
+        name = self._parse_type()
+        bases = []
+        if self.skip_string(':'):
+            self.skip_ws()
+            while 1:
+                access = 'private'
+                if self.match(_visibility_re):
+                    access = self.matched_text
+                base = self._parse_type()
+                bases.append(ClassDefExpr(base, access, False, []))
+                if self.skip_string(','):
+                    self.skip_ws()
+                else:
+                    break
+        return ClassDefExpr(name, visibility, static, bases)
 
     def read_rest(self):
         rv = self.definition[self.pos:]
@@ -891,6 +917,17 @@ class DefinitionParser(object):
 class CPPObject(ObjectDescription):
     """Description of a C++ language object."""
 
+    doc_field_types = [
+        GroupedField('parameter', label=l_('Parameters'),
+                     names=('param', 'parameter', 'arg', 'argument'),
+                     can_collapse=True),
+        GroupedField('exceptions', label=l_('Throws'), rolename='cpp:class',
+                     names=('throws', 'throw', 'exception'),
+                     can_collapse=True),
+        Field('returnvalue', label=l_('Returns'), has_arg=False,
+              names=('returns', 'return')),
+    ]
+
     def attach_name(self, node, name):
         owner, name = name.split_owner()
         varname = unicode(name)
@@ -913,8 +950,8 @@ class CPPObject(ObjectDescription):
         pnode += nodes.Text(text)
         node += pnode
 
-    def attach_modifiers(self, node, obj):
-        if obj.visibility != 'public':
+    def attach_modifiers(self, node, obj, visibility='public'):
+        if obj.visibility != visibility:
             node += addnodes.desc_annotation(obj.visibility,
                                              obj.visibility)
             node += nodes.Text(' ')
@@ -939,7 +976,7 @@ class CPPObject(ObjectDescription):
 
         indextext = self.get_index_text(name)
         if indextext:
-            self.indexnode['entries'].append(('single', indextext, theid, name))
+            self.indexnode['entries'].append(('single', indextext, theid, ''))
 
     def before_content(self):
         lastname = self.names and self.names[-1]
@@ -989,6 +1026,14 @@ class CPPClassObject(CPPObject):
         self.attach_modifiers(signode, cls)
         signode += addnodes.desc_annotation('class ', 'class ')
         self.attach_name(signode, cls.name)
+        if cls.bases:
+            signode += nodes.Text(' : ')
+            for base in cls.bases:
+                self.attach_modifiers(signode, base, 'private')
+                signode += nodes.emphasis(unicode(base.name),
+                                          unicode(base.name))
+                signode += nodes.Text(', ')
+            signode.pop()  # remove the trailing comma
 
 
 class CPPTypeObject(CPPObject):
@@ -1091,8 +1136,9 @@ class CPPFunctionObject(CPPObject):
 
 
 class CPPCurrentNamespace(Directive):
-    """This directive is just to tell Sphinx that we're documenting
-    stuff in namespace foo.
+    """
+    This directive is just to tell Sphinx that we're documenting stuff in
+    namespace foo.
     """
 
     has_content = False
@@ -1189,7 +1235,7 @@ class CPPDomain(Domain):
             env.warn_node('unparseable C++ definition: %r' % target, node)
             return None
 
-        parent = node['cpp:parent']
+        parent = node.get('cpp:parent', None)
 
         rv = _create_refnode(expr)
         if rv is not None or parent is None:
